# Build the minimal base image first (required by iowarp:latest)
docker build --no-cache -t iowarp/iowarp-base-minimal:latest -f iowarp-base-minimal.Dockerfile .
# docker push iowarp/iowarp-base-minimal:latest

docker build --no-cache -t iowarp/iowarp:latest -f iowarp.Dockerfile .
# docker push iowarp/iowarp:latest

docker build --no-cache -t iowarp/iowarp-build:latest -f iowarp-build.Dockerfile .
# docker push iowarp/iowarp-build:latest
